@@ -1,4 +1,4 @@
-/* @(#)timedefs.h	1.2 98/08/25 Copyright 1996 J. Schilling */
+/* @(#)timedefs.h	1.5 99/05/13 Copyright 1996 J. Schilling */
 /*
  *	Generic header for users of gettimeofday() ...
  *
@@ -27,10 +27,15 @@
 #include <mconfig.h>
 #endif
 
-#include <time.h>
-
+#ifdef	TIME_WITH_SYS_TIME_H
+#	include <sys/time.h>
+#	include <time.h>
+#else
 #ifdef	HAVE_SYS_TIME_H
-#include <sys/time.h>
+#	include <sys/time.h>
+#else
+#	include <time.h>
+#endif
 #endif
 
 #ifdef	__CYGWIN32__
@@ -43,12 +48,23 @@
 #endif
 #endif
 
+#ifdef	__EMX__
+/*
+ * EMX for OS/2 defines struct timeval in sys/time.h but not timerclear
+ */
+#ifndef	timerclear
+#define	timerclear(tvp)		(tvp)->tv_sec = (tvp)->tv_usec = 0
+#endif
+#endif
+
 #ifndef	timerclear
 
+#ifndef	VMS
 struct timeval {
 	long	tv_sec;
 	long	tv_usec;
 };
+#endif
 
 struct timezone {
 	int	tz_minuteswest;
