@@ -1,27 +1,14 @@
-/* @(#)io.h	2.9 98/09/05 Copyright 1986, 1995 J. Schilling */
+/* @(#)io.h	2.10 00/05/07 Copyright 1986, 1995 J. Schilling */
 /*
  *	Copyright (c) 1986, 1995 J. Schilling
  */
-/*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
- */
+/*@@C@@*/
 
 #include <mconfig.h>
 #include <standard.h>
 #include <unixstd.h>
 #include <fctldefs.h>
+#include <schily.h>
 
 #ifdef	NO_USG_STDIO
 #	ifdef	HAVE_USG_STDIO
