@@ -1,7 +1,7 @@
-/* @(#)scsi-bsd-os.c	1.14 00/02/06 Copyright 1997 J. Schilling */
+/* @(#)scsi-bsd-os.c	1.17 00/07/01 Copyright 1997 J. Schilling */
 #ifndef lint
 static	char __sccsid[] =
-	"@(#)scsi-bsd-os.c	1.14 00/02/06 Copyright 1997 J. Schilling";
+	"@(#)scsi-bsd-os.c	1.17 00/07/01 Copyright 1997 J. Schilling";
 #endif
 /*
  *	Interface for the BSD/OS user-land raw SCSI implementation.
@@ -52,7 +52,7 @@ static	char __sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  */
-LOCAL	char	_scg_trans_version[] = "scsi-bsd-os.c-1.14";	/* The version for this transport*/
+LOCAL	char	_scg_trans_version[] = "scsi-bsd-os.c-1.17";	/* The version for this transport*/
 
 #define	MAX_SCG		16	/* Max # of SCSI controllers */
 #define	MAX_TGT		16
@@ -181,8 +181,13 @@ openbydev:
 		if (device == NULL || device[0] == '\0')
 			return (0);
 		f = open(device, O_RDWR|O_NONBLOCK);
-		if (f < 0)
+		if (f < 0) {
+			if (scgp->errstr)
+				js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
+					"Cannot open '%s'",
+					device);
 			return (0);
+		}
 		if (tlun == -2) {	/* If 'lun' is not known, we reject */
 			close(f);
 			errno = EINVAL;
@@ -268,8 +273,9 @@ scsi_setup(scgp, f, busno, tgt, tlun)
 }
 
 LOCAL long
-scsi_maxdma(scgp)
+scsi_maxdma(scgp, amt)
 	SCSI	*scgp;
+	long	amt;
 {
 	long maxdma = MAX_DMA_BSDI;
 
@@ -281,7 +287,7 @@ scsi_getbuf(scgp, amt)
 	SCSI	*scgp;
 	long	amt;
 {
-	if (amt <= 0 || amt > scsi_maxdma(scgp))
+	if (amt <= 0 || amt > scsi_bufsize(scgp, amt))
 		return ((void *)0);
 	if (scgp->debug)
 		printf("scsi_getbuf: %ld bytes\n", amt);
